
indexing
	description: "Variant of DS_HASH_TABLE indexed by NXML_NAME, with namespace context"
	
class NXML_NAME_TABLE [G]

inherit
	DS_HASH_TABLE [G, NXML_NAME]
		rename
			make as hash_table_make
		end
	
creation
	make
	
feature {NONE} -- Creation

	make (a_namespace: NXML_NAMESPACE) is
			-- Namespace.
		require
			not_void: a_namespace /= Void
		do
			namespace := a_namespace
			make_default
		ensure
			keep_reference: namespace = a_namespace
		end
		
	namespace: NXML_NAMESPACE

feature -- Named interface to DS_HASH_TABLE
	
	has_named (a_string: STRING): BOOLEAN is
			-- Is the name present in this table?
		require
			name: a_string /= Void
		do
			Result := has (name (a_string))
		ensure
			def: Result = has (name (a_string))
		end
		
	named (a_string: STRING): G is
			-- Named.
		require
			has: has_named (a_string)
		do
			Result := item (name (a_string))
		ensure
			def: Result = item (name (a_string))
		end
		
	force_named (v: G; a_string: STRING) is
			-- Force with name.
			-- Replace element if key already used.
		require
			name: a_string /= Void
		do
			force (v, name (a_string))
		ensure
			forced: has_named (a_string)
			-- as `force'.
		end
		
	force_new_named (v: G; a_string: STRING) is
			-- Force new with name.
		require
			name: a_string /= Void
			has: has_named (a_string)
		do
			force_new (v, name (a_string))
		ensure
			forced: has_named (a_string)
			-- as `force_new'.
		end
		
feature {NONE} -- Implementation

	name (a_string: STRING): NXML_NAME is
		do
			-- could use sharing?
			!! Result.make (namespace, a_string)
		ensure
			not_void: Result /= Void
		end
		
invariant
	namespace: namespace /= Void
	
end
