
indexing
	description: "Namespace URI"
	aspects: immutable
	
class NXML_NAMESPACE

inherit
	HASHABLE
		redefine
			is_equal,
			out
		end

creation
	make
		
feature {NONE} -- Creation

	make (an_uri: STRING) is
			-- Create.
		require
			not_void: an_uri /= Void
		do
			uri := an_uri
		end
		
feature -- HASHABLE

	hash_code: INTEGER is
			-- Hash code.
		do
			Result := uri.hash_code
		end

feature -- Output
	
	out: STRING is
			-- URI.
		do
			Result := uri
		ensure then
			definition: Result = uri
		end
		
	as_xml (a_ns_map: DS_SPARSE_TABLE [STRING, NXML_NAMESPACE]): STRING is
			-- Resolved prefix.
		require
			has: a_ns_map.has (Current)
		do
			Result := a_ns_map.item (Current)
		end

	namespace_declaration: STRING is
			-- xmlns=<uri>
		do
			!! Result.make (0)
			namespace_declaration_in (Result)
		ensure
			new_string: Result /= Void
		end
		
	namespace_declaration_in (a_string: STRING) is
			-- Append `namespace_declaration' to string.
		require
			appendable: a_string /= Void
		do
			a_string.append_string (Xmlns)
			append_equal_uri_in (a_string)
		end
	
	namespace_prefix_declaration (a_name: STRING): STRING is
			-- xmlns:<a_name>='<uri>'
		require
			is_prefix: is_namespace_prefix (a_name)
		do
			!! Result.make (0)
			namespace_prefix_declaration_in (Result, a_name)
		ensure
			new_string: Result /= Void
		end
		
	namespace_prefix_declaration_in (a_string: STRING; a_name: STRING) is
			-- Append `named_namespace_declaration' to string.
		require
			appendable: a_string /= Void
			is_prefix: is_namespace_prefix (a_name)
		do
			a_string.append_string (Xmlns)
			a_string.append_character (':')
			a_string.append_string (a_name)
			append_equal_uri_in (a_string)
		end
		
	is_namespace_prefix (a_string: STRING): BOOLEAN is
			-- Is this string a valid namespace prefix.
		do
			Result := a_string /= Void
				and then a_string.count > 0
				-- and has valid characters...
		end
					
feature {NONE} -- Output implementation
	
	append_equal_uri_in (a_string: STRING) is
			-- Append ='<uri>' to string.
		require
			appendable: a_string /= Void
		do
			a_string.append_character ('=')
			a_string.append_character ('%'')
			a_string.append_string (uri)
			a_string.append_character ('%'')
		end
		
	Xmlns: STRING is 
			-- Namespace declaration prefix.
		once
			Result := "xmlns"
		end
			
feature -- ANY
	
	is_equal (an_other: like Current): BOOLEAN is
			-- Is equal?
		do
			Result := (an_other = Current)
				or else an_other.uri = uri
				or else an_other.uri.is_equal (uri)
		end
		
feature -- Data

	uri: STRING
			-- Namespace unique identifier.

invariant
	not_void_uri: uri /= Void
	
end
