indexing
	description: "Test parser - output some pseudo-canonical XML"
	project: "http://www.nenie.org/eiffel/"
	license: "http://www.opensource.org/licenses/mit-license.html"
	revision: "$Id: nxml_parser_canonical.e,v 1.1 2001/07/09 23:15:38 farnaud Exp $"
	
class
	NXML_PARSER_CANONICAL

inherit
	NXML_PARSER_PEDANTIC
		redefine
			when_comment,
			when_processing_instruction,
			when_start_tag,
			when_start_tag_attribute,
			when_start_tag_end,
			when_content,
			when_end_tag
		end
		
	
creation
	make

feature -- Events (introduced here)

	when_output_string (a_string: STRING) is
			-- Output string.
		require
			not_void: a_string /= Void
		do
			io.put_string (a_string)
		end
		
feature -- Events

	when_comment (a_content: STRING) is
			-- Comment.
		do
			Precursor (a_content)
			-- No comment in first canon form
		end

	when_processing_instruction (a_target, a_content: STRING) is
			-- PI.
		do
			Precursor (a_target, a_content)
			when_output_string ("<?")
			when_output_string (a_target)
			when_output_string (" ")
			when_output_string (a_content)
			when_output_string ("?>")
		end

	when_start_tag (a_name: NXML_PARSER_NAME) is
			-- Start tag.
		do
			Precursor (a_name)
			when_output_string ("<")
			when_output_string (a_name.out)
		end
		
	when_start_tag_attribute (a_name: NXML_PARSER_NAME; a_value: STRING) is
			-- Attribute
		do
			Precursor (a_name, a_value)
			when_output_string (" ")
			when_output_string (a_name.out)
			when_output_string ("=%"")
			when_output_string (escape (a_value))
			when_output_string ("%"")
		end
		
	when_start_tag_end is
			-- End of start tag.
		do
			Precursor
			when_output_string (">")
		end

	when_content (a_string: STRING) is
			-- Text.
		do
			Precursor (a_string)
			when_output_string (escape (a_string))
		end

	when_end_tag (a_name: NXML_PARSER_NAME) is
			-- End tag.
		do
			Precursor (a_name)
			when_output_string ("</")
			when_output_string (a_name.out)
			when_output_string (">")
		end

feature {NONE} -- Implementation
		
	escape (a_string: STRING): STRING is
			-- Escape XML chars in content for 'canonical' output.
		local
			i, cnt: INTEGER
			a_char: CHARACTER
		do
			from
				Result := a_string
				i := 1
				cnt := a_string.count
			until
				i > cnt
			loop
				a_char := a_string.item (i)
				if a_char = '<' 
						or a_char = '&' 
						or a_char = '>' 
						or a_char =	'%"'
				then
					-- create string if no escaped char till now
					if Result = a_string then
						if i = 1 then
							!! Result.make (0)
						else
							Result := a_string.substring (1,i-1)
						end
					end
					check new_result: Result /= a_string end
					inspect
						a_char
					when '<' then
						Result.append_string ("&lt;")
					when '&' then
						Result.append_string ("&amp;")
					when '>' then
						Result.append_string ("&quot;")
					when '%"' then
						Result.append_string ("&amp;")
					end
					
				elseif Result /= a_string then
					check new_result: Result /= a_string end
					Result.append_character (a_char)
				end
				i := i + 1
			end
		end
		
end
