indexing
	description: "Parser that produces a NXML_ELEMENT based tree -- no mixed content"
	project: "http://www.nenie.org/eiffel/"
	
class
	NXML_PARSER_ELEMENT

inherit
	NXML_PARSER_NAMESPACE

creation
	make
		
feature -- Events

	when_start (a_name: NXML_NAME) is
			-- Start tag.
		do
			-- prepare name
			if last_element = Void then
				!! last_element.make_root (a_name)
			else
				!! last_element.make_child (last_element, a_name)
				last_element.parent.add_element (last_element)
			end
			
			-- recipient of content if any
			!! last_text.make (0)
		end

	when_attribute (a_name: NXML_NAME; a_value: STRING) is
			-- Attribute.
		do
			last_element.attributes.force (a_value, a_name)
		end
		
	when_content_start is
			-- Do nothing.
		do
		end
		
	when_content (a_string: STRING) is
			-- Text.
		do
			-- only collect text if no mixed content
			if last_element.is_leaf then
				last_text.append_string (a_string)
			end
		end

	when_end is
			-- End tag.
		do
			-- only collect text if no mixed content
			if last_element.is_leaf then
				last_element.set_text (last_text)
			end
			
			-- pop
			if not last_element.is_root then
				last_element := last_element.parent
			end
		end

	when_error (a_message: STRING) is
			-- Report error message.
		do
			std.error.put_string (error_header)
			std.error.put_string (a_message)
			std.error.put_string ("%N")
		end

feature -- Access

	last_element: NXML_ELEMENT
			-- Last element.
			-- Root node when parsing finished and successful.
		
feature {NONE} -- Implementation

	last_text: STRING
	
end
