
indexing
	description: "XML parser variant doing unicode character class validation"
	project: "http://www.nenie.org/eiffel/"
	license: "http://www.opensource.org/licenses/mit-license.html"
	revision: "$Id: nxml_parser_pedantic.e,v 1.2 2001/07/13 20:35:51 farnaud Exp $"
	-- TODO
	-- validation of characters within DTDs?
	
class NXML_PARSER_PEDANTIC

inherit
	NXML_PARSER
		redefine
			new_literal_entity
		end

	NXML_EVENT_INTERFACE
		-- deferred routines

	NXML_CHARACTER_CLASSES
		export
			{NONE} all
		end
	
	NXML_UTF8_PRIVATE_ROUTINES
		
feature -- Actions (redirected)

	when_comment (a: STRING) is
		do
			validate (a)
		end
	
	when_processing_instruction (a,b: STRING) is
		do
			validate_string_name (a)
			validate (b)
		end
	

	when_start_tag (a: NXML_PARSER_NAME) is
			-- Start tag.
		do
			validate_name (a)
		end
		
	when_start_tag_attribute (a: NXML_PARSER_NAME; a_value: STRING) is
			-- Attribute
		do
			validate_name (a)
			validate (a_value)
		end

	when_start_tag_end is
			-- End of start tag.
		do
		end
		
	when_content (a: STRING) is
			-- Text.
		do
			validate (a)
		end

	when_end_tag (a: NXML_PARSER_NAME) is
			-- End tag.
		do
			validate_name (a)
		end

	when_error (a_message: STRING) is
			-- Report error message to standard 
			-- output.
		do
			std.error.put_string (error_header)
			std.error.put_string (a_message)
			std.error.put_string ("%N")
		end

feature {NONE} -- Actions (private)

	new_literal_entity (a_value: STRING): NXML_ENTITY_DEF is
			-- New literal entity definition.
			-- This is not part of the public interface.
			-- It catches invalid XML characters in unused but 
			-- declared entities.
		do
			validate (a_value)
			Result := Precursor (a_value)
		end


feature {NONE}

	validate (a:STRING) is
			-- Validate a string.
		do	
			if not valid_string (a)  then	
				force_error (Error_unicode_invalid_character)
			end
		end
		
	validate_name (a:NXML_PARSER_NAME) is
			-- Validate a string.
		do	
			validate_string_name (a.out)
		end

	validate_string_name (a:STRING) is
			-- Validate name in astring
		do
			if not valid_name (a)  then	
				force_error (Error_unicode_invalid_character)
			end
		end
		
feature {NONE} -- String

	valid_string (a: STRING): BOOLEAN is
			-- Is this a UTF8 string with no invalid character?
		require
			not_void: a /= Void
		local
			it: NXML_UTF8_CURSOR
		do
			if is_utf8_string (a) then
				from
					Result := True
					!! it.make (a)
					it.start
				until
					it.after or not Result
				loop
					Result := is_char (it.item)
					it.forth
				end
			end
		ensure
			empty: (a.count = 0) implies Result
		end
		
	valid_name (a: STRING): BOOLEAN is
			-- Is this a UTF8 string with a valid name?
		require
			not_void: a /= Void
		local
			it: NXML_UTF8_CURSOR
		do
			if is_utf8_string (a) then
				!! it.make (a)
				it.start

				-- first
				Result := it.after or else is_name_first (it.item)

				-- tail
				from until
					it.after or not Result
				loop
					Result := is_name_char (it.item)
					it.forth
				end
			end
		end
		
end
