indexing
	description: "Test driver for Nenie XML"
	license: "http://www.opensource.org/licenses/mit-license.html"
	revision: "$Id: nxml_test.e,v 1.7 2001/07/14 00:19:45 farnaud Exp $"
	
class NXML_TEST

inherit
	ANY
	ARGUMENTS

creation
	make

feature -- Action

	make is
			-- Parse XML files `arguments (1..argument_count)'.
		local
			j: INTEGER
			an_arg: STRING
		do
			if argument_count = 0 then
				usage
			else
				from
					j := 1
				until
					j > argument_count 
				loop
					an_arg := argument (j)
					if an_arg.is_equal ("--help") then
						usage
						
					-- toggle options
					elseif an_arg.is_equal ("--tree") then
						tree := not tree
					elseif an_arg.is_equal ("--no-errors") then
						no_errors := not no_errors
					elseif an_arg.is_equal ("--expect-valid") then
						expect_valid := not expect_valid
					elseif an_arg.is_equal ("--expect-invalid") then
						expect_invalid := not expect_invalid
					elseif an_arg.is_equal ("--quiet") then
						quiet := not quiet
					elseif an_arg.is_equal ("--header") then
						header := not header
					-- file
					else
						process_file (an_arg)
					end
					j := j + 1
				end
			end
		end

feature -- Options

	tree: BOOLEAN
			-- Use tree parser.
	quiet: BOOLEAN
			-- Do not display canonical XML on standard output.
	no_errors: BOOLEAN
			-- Do not display parser's error message.
	header: BOOLEAN
			-- Display name of file being processed.
	expect_valid: BOOLEAN
			-- Expect valid document.
	expect_invalid: BOOLEAN
			-- Expect invalid document.
			
feature {NONE} -- Implementation

	process_file (an_arg: STRING) is
			-- Process a file.
		require
			not_void: an_arg /= Void
		local
			parser: NXML_PARSER
			element_parser: NXML_PARSER_ELEMENT
			test_parser: NXML_PARSER_TEST
		do
			-- title
			if header then
				io.put_string (an_arg)
				io.put_new_line
			end
					
			-- parse
			if tree then
				!!element_parser.make
				element_parser.parse_file (an_arg)
				if not element_parser.syntax_error then
					io.put_string
						(element_parser.last_element.as_xml						
							(element_parser.last_element.default_namespaces_map))
				end
				parser := element_parser
			else
				!! test_parser.make
				
				test_parser.set_quiet (quiet)
				test_parser.set_hide_errors (no_errors)

				parser := test_parser
				parser.parse_file (an_arg)
			end

			-- display result if desired
			if expect_valid and parser.syntax_error then
				io.put_string (an_arg)
				io.put_string (": INVALID")
				io.put_new_line
			end
			if expect_invalid and not parser.syntax_error then
				io.put_string (an_arg)
				io.put_string (": UNEXPECTEDLY VALID")
				io.put_new_line
			end
		end
		
feature {NONE} -- Help

	usage is
			-- Usage
		do
			io.put_string ("usage: nxml (--<toggle-option> | filename)+")
			io.put_new_line
			io.put_string ("  toggle options: --tree --expect-valid --expect-invalid")
			io.put_new_line
		end
end

