
indexing
	description: "Variant of YY_FILE_BUFFER that accepts UTF16 and converts it to UTF8"
	project: "http://www.nenie.org/eiffel/"
	license: "http://www.opensource.org/licenses/mit-license.html"
	revision: "$Id: nxml_utf16_file_buffer.e,v 1.4 2001/07/14 00:19:45 farnaud Exp $"
	--xml_specific_zero: "Zero characters conflict with EOB but are not valid XML"
	--xml_specific_lt: "Detection assuming first char is < is XML specific"
	-- TODO:
	-- close file OK?
	-- efficient fill
	-- factorise UTF8 encoding routines
	
class NXML_UTF16_FILE_BUFFER

inherit
	YY_FILE_BUFFER
		redefine
			set_file,
			fill,
			resize
		end

	NXML_UTF8_PRIVATE_ROUTINES
		
creation
	make, make_with_size
	
feature

	set_file (a_file: like INPUT_STREAM_TYPE) is
			-- Set file associated with this buffer.
			-- THE FILE WILL BE CLOSED by this class.
		do
			Precursor (a_file)
			is_detected := False
			is_utf16 := False
		end
		
	fill is
			-- Currently inefficiently fills one character at a time...
		do
			if not is_detected then
				-- detect UTF16
				is_detected := True
				check filled: filled end
				
				read_double_char
				if has_double_char then
					-- check byte ordering character
					if (first_char.code = 254 and second_char.code = 255) then
						is_utf16 := True
						is_msb_first := True
					elseif (first_char.code = 255 and second_char.code = 254) then
						is_utf16 := True
						is_msb_first := False
					-- check wide '<'
					elseif (first_char.code = 0 and second_char = '<') then 
						is_utf16 := True
						is_msb_first := False
						append_lt
					elseif (first_char = '<' and second_char.code = 0) then
						is_utf16 := True
						is_msb_first := False
						append_lt
					-- not unicode
					else
						upper := upper + 1
						content.put (first_char, upper)
						upper := upper + 1
						content.put (second_char, upper)
						-- Precursor will add EOB
					end			
				end
			end
				
			if is_utf16 then
				if filled and not end_of_file then
					compact_left
					read_double_char
					filled := has_double_char
					if filled then
						if is_msb_first then
							append_character (first_char.code * 256 + second_char.code)
						else
							append_character (second_char.code * 256 + first_char.code)
						end
						
						-- EOB
						content.put (End_of_buffer_character, upper + 1)
						content.put (End_of_buffer_character, upper + 2)
					end
				end
			else
				-- not UTF16, use normal processing
				Precursor
			end
			
			-- close file
			if end_of_file and not INPUT_STREAM_.is_closed (file) then
				INPUT_STREAM_.close (file)
			end
		end
		
feature {NONE} -- Implementation

	append_lt is
			-- Append '<'.
		do
			upper := upper + 1
			content.put ('<', upper)
		end
		
	append_character (a_char: INTEGER) is
			-- Append character from UTF16 code.
		require
			positive: a_char >= 0
			max_utf16: a_char < 65536
		local
			i: INTEGER
			enc_size: INTEGER
		do
			debug ("nxml")
				io.put_string ("[")
				io.put_string (a_char.out)
				io.put_string ("]")
			end

			-- convert to UTF8
			from
				enc_size := encoding_size (a_char)
				i := 1
			until
				i > enc_size
			loop
				upper := upper + 1
				content.put (encoded_octet (a_char, i, enc_size), upper)
				
				i := i + 1
			end
		end
		
feature {NONE} -- Implementation

	resize is
			-- Increase `capacity' so that there is always 
			-- space for one 16-bit UTF8 char (3 chars) plus 
			-- 2 detection chars.
		do
			if capacity = 0 then
        		capacity := Default_capacity
			else
        		capacity := capacity * 2
			end

			-- Include room for 2 EOB characters.
			if capacity + 10 > content.count then
        	        		-- Set `content.count' to `capacity' + 2.
        		content := STRING_BUFFER_.resize (content, capacity + 5)
			end
		end

feature {NONE} -- Reading
		
	read_double_char is
			-- Read two characters from file.
		do
			first_char := '%/0/'
			second_char := '%/0/'
			has_double_char := False
			
			file.read_character
			end_of_file := INPUT_STREAM_.end_of_input (file)
			if not end_of_file then
				first_char := file.last_character

				file.read_character
				end_of_file := INPUT_STREAM_.end_of_input (file)
				if not end_of_file then
					second_char := file.last_character
					has_double_char := True
				end
			end
		end
		
	has_double_char: BOOLEAN
	first_char: CHARACTER
	second_char: CHARACTER
		
feature -- State

	is_utf16: BOOLEAN
			-- Are we reading an UTF16 file?

feature {NONE} -- State

	is_msb_first: BOOLEAN
			-- Is the most significant byte first?
	
	is_detected: BOOLEAN
			-- Have first two bytes been read?
end
