
indexing
	description: "Cursor on a STRING that happens to contain a UTF8 string"
	implements_gobo_cursor: "Implements DS_LINEAR_CURSOR[INTEGER] interface"
	license: "http://www.opensource.org/licenses/mit-license.html"
	revision: "$Id: nxml_utf8_cursor.e,v 1.4 2001/07/14 00:19:45 farnaud Exp $"
		-- Does not inherit because STRING is not a Gobo container, 
		-- and the item is not really a STRING's item (CHARACTER)
	
class NXML_UTF8_CURSOR

inherit
	ANY
	NXML_UTF8_PRIVATE_ROUTINES

creation
	make
	
feature {NONE} -- Creation

	make (a_string: STRING) is
			-- Associate an UTF-8 string with this cursor.
			-- `start' will go `after' if the string is not
			-- a well formed UTF-8 string.
		require
			not_void: a_string /= Void
		do
			container := a_string
			position := 0
		end
		
feature {NXML_UTF8_CURSOR} -- State

	position: INTEGER

feature -- Access

	item: INTEGER is
			-- Character code for current character
		require
			not_off: not off
		local
			code_at_position: INTEGER
			i, cnt: INTEGER
		do
			code_at_position := container.item (position).code
			
			from
				i := 2
				cnt := nr_of_encoding_bytes (code_at_position)
				Result := decoded_first (code_at_position, cnt)
			variant
				cnt + 1 - i
			until
				i > cnt
			loop
				Result := Result * 64 -- shift 6 bits
				Result := Result + decoded_following (
							container.item (position + i - 1).code)
				
				i := i + 1
			end
		end
				 
	container: STRING
			-- String traversed
	  
feature -- Status

	off: BOOLEAN is
			-- Is there no item at cursor position?
		do
			Result := (position < 1) or (position > container.count)
		end
		
	same_position (other: like Current): BOOLEAN is
			-- Is current cursor at same position as `other'?
		require
			other_not_void: other /= Void
		do
			Result := (other.container = container)
				and (other.position = position)
		end
		
feature -- Status report

	is_first: BOOLEAN is
			-- Is cursor on first item?
		do
			Result := position = 1
		ensure
			not_empty: Result implies container.count > 0
			not_off: Result implies not off
		end

	after: BOOLEAN is
			-- Is there no valid position to right of cursor?
		do
			Result := position > container.count
		ensure
			after_constraint: Result implies off
		end
		
feature	-- Cursor movement

	start is
			-- Move cursor to first position.
		do
			if is_utf8_string (container) then
				position := 1
			else
				go_after
			end	
		ensure
			empty_behavior: (container.count = 0) implies after
			not_empty_behavior: ((container.count > 0) and is_utf8_string (container)) 
				implies is_first
			not_utf8: not is_utf8_string (container) implies after
		end
		
	forth is
			-- Move cursor to next position.
		require
			not_after: not after
		do
			position := position + nr_of_encoding_bytes (container.item (position).code)
		end
		
	search_forth (v: INTEGER) is
			-- Move to first position at or after current
			-- position where `item' and `v' are equal.
		require
			not_off: not off or after
		do
			from
			until
				after or else (item = v)
			loop
				forth
			end
		ensure
			done: after or else (item = v)
		end
		
	go_after is
			-- Move cursor to `after' position.
		do
			position := container.count + 1 -- after
		ensure
			after: after
		end
		
invariant
	container_not_void: container /= Void
 
end
